@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2001, 2002, 2003 Tama Communications Corporation
+ * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -236,6 +236,10 @@ C(yacc)
 				warning("'%%}' appeared in Yacc mode. [+%d %s].", lineno, curfile);
 			inC = 0;
 			break;
+		case YACC_UNION:	/* %union {...} */
+			if (yaccstatus == DECLARATIONS && target == DEF)
+				PUT("YYSTYPE", lineno, sp);
+			break;
 		/*
 		 * #xxx
 		 */
@@ -265,12 +269,15 @@ C(yacc)
 				}
 			}
 			break;
+		case SHARP_IMPORT:
 		case SHARP_INCLUDE:
+		case SHARP_INCLUDE_NEXT:
 		case SHARP_ERROR:
 		case SHARP_LINE:
 		case SHARP_PRAGMA:
 		case SHARP_WARNING:
 		case SHARP_IDENT:
+		case SHARP_SCCS:
 			while ((c = nexttoken(interested, reserved_word)) != EOF && c != '\n')
 				;
 			break;
@@ -565,7 +572,8 @@ function_definition(target, arg1)
 			brace_level++;
 		else if (c == /* ( */')' || c == ']')
 			brace_level--;
-		else if (brace_level == 0 && (c == SYMBOL || IS_RESERVED(c)))
+		else if (brace_level == 0
+		    && ((c == SYMBOL && strcmp(token, "__THROW")) || IS_RESERVED_WORD(c)))
 			isdefine = 1;
 		else if (c == ';' || c == ',') {
 			if (!isdefine)
