@@ -1,8 +1,7 @@
 /*
- * Copyright (c) 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001, 2002
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1998, 1999 Shigio Yamaguchi
+ * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -38,7 +37,6 @@
 #include <strings.h>
 #endif
 
-#include "Cpp.h"
 #include "gctags.h"
 #include "defined.h"
 #include "die.h"
@@ -46,11 +44,16 @@
 #include "strbuf.h"
 #include "strlimcpy.h"
 #include "token.h"
+#include "cpp_res.h"
 
-static	int	function_definition(int);
-static  int     seems_datatype(const char *);
-static	void	condition_macro(int);
-static	int	reserved(char *);
+static void process_attribute(int);
+static int function_definition(int);
+static int seems_datatype(const char *);
+static void condition_macro(int, int);
+
+#define MAXCOMPLETENAME 1024            /* max size of complete name of class */
+#define MAXCLASSSTACK   100             /* max size of class stack */
+#define IS_CV_QUALIFIER(c)      ((c) == CPP_CONST || (c) == CPP_VOLATILE)
 
 /*
  * #ifdef stack.
@@ -64,6 +67,7 @@ static struct {
 } pifstack[MAXPIFSTACK], *cur;
 static int piflevel;		/* condition macro level */
 static int level;		/* brace level */
+static int namespacelevel;	/* namespace block level */
 
 /*
  * Cpp: read C++ file and pickup tag entries.
@@ -71,26 +75,26 @@ static int level;		/* brace level */
 void
 Cpp()
 {
-	int	c, cc;
-	int	savelevel;
-	int	target;
-	int	startclass, startthrow, startmacro, startsharp, startequal;
-	char    classname[MAXTOKEN];
-	char    completename[MAXCOMPLETENAME];
-	int     classlevel;
+	int c, cc;
+	int savelevel;
+	int target;
+	int startclass, startthrow, startmacro, startsharp, startequal;
+	char classname[MAXTOKEN];
+	char completename[MAXCOMPLETENAME];
+	int classlevel;
 	struct {
 		char *classname;
 		char *terminate;
-		int   level;
+		int level;
 	} stack[MAXCLASSSTACK];
-	const	char *interested = "{}=;~";
-	STRBUF	*sb = strbuf_open(0);
+	const char *interested = "{}=;~";
+	STRBUF *sb = strbuf_open(0);
 
 	*classname = *completename = 0;
 	stack[0].classname = completename;
 	stack[0].terminate = completename;
 	stack[0].level = 0;
-	level = classlevel = piflevel = 0;
+	level = classlevel = piflevel = namespacelevel = 0;
 	savelevel = -1;
 	target = (sflag) ? SYM : (rflag) ? REF : DEF;
 	startclass = startthrow = startmacro = startsharp = startequal = 0;
@@ -98,7 +102,7 @@ Cpp()
 	crflag = 1;			/* require '\n' as a token */
 	cppmode = 1;			/* treat '::' as a token */
 
-	while ((cc = nexttoken(interested, reserved)) != EOF) {
+	while ((cc = nexttoken(interested, reserved_word)) != EOF) {
 		if (cc == '~' && level == stack[classlevel].level)
 			continue;
 		switch (cc) {
@@ -111,11 +115,16 @@ Cpp()
 					if (target == REF && defined(token))
 						PUT(token, lineno, sp);
 				} else if (level > stack[classlevel].level || startequal || startmacro) {
-					if (target == REF && defined(token))
-						PUT(token, lineno, sp);
+					if (target == REF) {
+						if (defined(token))
+							PUT(token, lineno, sp);
+					} else if (target == SYM) {
+						if (!defined(token))
+							PUT(token, lineno, sp);
+					}
 				} else if (level == stack[classlevel].level && !startmacro && !startsharp && !startequal) {
-					char	savetok[MAXTOKEN], *saveline;
-					int	savelineno = lineno;
+					char savetok[MAXTOKEN], *saveline;
+					int savelineno = lineno;
 
 					strlimcpy(savetok, token, sizeof(savetok));
 					strbuf_reset(sb);
@@ -145,9 +154,40 @@ Cpp()
 				}
 			}
 			break;
+		case CPP_USING:
+			if ((c = nexttoken(interested, reserved_word)) == CPP_NAMESPACE) {
+				if ((c = nexttoken(interested, reserved_word)) == SYMBOL) {
+					if (target == REF)
+						PUT(token, lineno, sp);
+				} else {
+					if (wflag)
+						warning("missing namespace name. [+%d %s].", lineno, curfile);
+					pushbacktoken();
+				}
+			} else
+				pushbacktoken();
+			/* FALLTHROUGH */
+		case CPP_NAMESPACE:
+			if (peekc(0) != '{') /* } */ {
+				if ((c = nexttoken(interested, reserved_word)) == SYMBOL)
+					if (target == DEF)
+						PUT(token, lineno, sp);
+			}
+			while ((c = nexttoken(interested, reserved_word)) == '\n')
+				;
+			/*
+			 * Namespace block doesn't have any influence on level.
+			 */
+			if (c == '{') /* } */ {
+				namespacelevel++;
+			} else {
+				if (wflag)
+					warning("missing namespace block. [+%d %s](0x%x).", lineno, curfile, c);
+			}
+			break;
 		case CPP_CLASS:
 			DBG_PRINT(level, "class");
-			if ((c = nexttoken(interested, reserved)) == SYMBOL) {
+			if ((c = nexttoken(interested, reserved_word)) == SYMBOL) {
 				strlimcpy(classname, token, sizeof(classname));
 				if (target == DEF)
 					PUT(token, lineno, sp);
@@ -160,7 +200,7 @@ Cpp()
 			++level;
 			if (bflag && atfirst) {
 				if (wflag && level != 1)
-					fprintf(stderr, "Warning: forced level 1 block start by '{' at column 0 [+%d %s].\n", lineno, curfile); /* } */
+					warning("forced level 1 block start by '{' at column 0 [+%d %s].", lineno, curfile); /* } */
 				level = 1;
 			}
 			if (startclass) {
@@ -183,13 +223,16 @@ Cpp()
 			/* { */
 		case '}':
 			if (--level < 0) {
-				if (wflag)
-					fprintf(stderr, "Warning: missing left '{' [+%d %s].\n", lineno, curfile); /* } */
+				if (namespacelevel > 0)
+					namespacelevel--;
+				else if (wflag)
+					warning("missing left '{' [+%d %s].", lineno, curfile); /* } */
 				level = 0;
 			}
 			if (eflag && atfirst) {
-				if (wflag && level != 0) /* { */
-					fprintf(stderr, "Warning: forced level 0 block end by '}' at column 0 [+%d %s].\n", lineno, curfile);
+				if (wflag && level != 0)
+					/* { */
+					warning("forced level 0 block end by '}' at column 0 [+%d %s].", lineno, curfile);
 				level = 0;
 			}
 			if (level < stack[classlevel].level)
@@ -206,7 +249,7 @@ Cpp()
 		case '\n':
 			if (startmacro && level != savelevel) {
 				if (wflag)
-					fprintf(stderr, "Warning: different level before and after #define macro. reseted. [+%d %s].\n", lineno, curfile);
+					warning("different level before and after #define macro. reseted. [+%d %s].", lineno, curfile);
 				level = savelevel;
 			}
 			startmacro = startsharp = 0;
@@ -214,17 +257,18 @@ Cpp()
 		/*
 		 * #xxx
 		 */
-		case CP_DEFINE:
+		case SHARP_DEFINE:
+		case SHARP_UNDEF:
 			startmacro = 1;
 			savelevel = level;
-			if ((c = nexttoken(interested, reserved)) != SYMBOL) {
+			if ((c = nexttoken(interested, reserved_word)) != SYMBOL) {
 				pushbacktoken();
 				break;
 			}
 			if (peekc(1) == '('/* ) */) {
 				if (target == DEF)
 					PUT(token, lineno, sp);
-				while ((c = nexttoken("()", reserved)) != EOF && c != '\n' && c != /* ( */ ')')
+				while ((c = nexttoken("()", reserved_word)) != EOF && c != '\n' && c != /* ( */ ')')
 					if (c == SYMBOL && target == SYM)
 						PUT(token, lineno, sp);
 				if (c == '\n')
@@ -239,40 +283,36 @@ Cpp()
 				}
 			}
 			break;
-		case CP_INCLUDE:
-		case CP_ERROR:
-		case CP_LINE:
-		case CP_PRAGMA:
-		case CP_WARNING:
-		case CP_IDENT:
-			while ((c = nexttoken(interested, reserved)) != EOF && c != '\n')
+		case SHARP_IMPORT:
+		case SHARP_INCLUDE:
+		case SHARP_INCLUDE_NEXT:
+		case SHARP_ERROR:
+		case SHARP_LINE:
+		case SHARP_PRAGMA:
+		case SHARP_WARNING:
+		case SHARP_IDENT:
+		case SHARP_SCCS:
+			while ((c = nexttoken(interested, reserved_word)) != EOF && c != '\n')
 				;
 			break;
-		case CP_IFDEF:
-		case CP_IFNDEF:
-		case CP_IF:
-		case CP_ELIF:
-		case CP_ELSE:
-		case CP_ENDIF:
-		case CP_UNDEF:
-			condition_macro(cc);
-			while ((c = nexttoken(interested, reserved)) != EOF && c != '\n') {
-				if (!strcmp(token, "defined"))
-					continue;
-				if (c == SYMBOL && target == SYM)
-					PUT(token, lineno, sp);
-			}
+		case SHARP_IFDEF:
+		case SHARP_IFNDEF:
+		case SHARP_IF:
+		case SHARP_ELIF:
+		case SHARP_ELSE:
+		case SHARP_ENDIF:
+			condition_macro(cc, target);
 			break;
-		case CP_SHARP:		/* ## */
-			(void)nexttoken(interested, reserved);
+		case SHARP_SHARP:		/* ## */
+			(void)nexttoken(interested, reserved_word);
 			break;
 		case CPP_NEW:
-			if ((c = nexttoken(interested, reserved)) == SYMBOL)
+			if ((c = nexttoken(interested, reserved_word)) == SYMBOL)
 				if (target == REF && defined(token))
 					PUT(token, lineno, sp);
 			break;
 		case CPP_STRUCT:
-			c = nexttoken(interested, reserved);
+			c = nexttoken(interested, reserved_word);
 			if (c == '{' /* } */) {
 				pushbacktoken();
 				break;
@@ -281,16 +321,11 @@ Cpp()
 				if (target == SYM)
 					PUT(token, lineno, sp);
 			break;
-		case CPP_EXTERN:
-			if (level > stack[classlevel].level && target == REF)
-				while ((c = nexttoken(";", reserved)) != EOF && c != ';')
-					;
-			break;
 		case CPP_TEMPLATE:
 			{
 				int level = 0;
 
-				while ((c = nexttoken("<>", reserved)) != EOF) {
+				while ((c = nexttoken("<>", reserved_word)) != EOF) {
 					if (c == '<')
 						++level;
 					else if (c == '>') {
@@ -302,11 +337,11 @@ Cpp()
 					}
 				}
 				if (c == EOF && wflag)
-					fprintf(stderr, "Warning: templete <...> isn't closed. [+%d %s].\n", lineno, curfile);
+					warning("templete <...> isn't closed. [+%d %s].", lineno, curfile);
 			}
 			break;
 		case CPP_OPERATOR:
-			while ((c = nexttoken(";{", /* } */ reserved)) != EOF) {
+			while ((c = nexttoken(";{", /* } */ reserved_word)) != EOF) {
 				if (c == '{') /* } */ {
 					pushbacktoken();
 					break;
@@ -318,7 +353,7 @@ Cpp()
 				}
 			}
 			if (c == EOF && wflag)
-				fprintf(stderr, "Warning: '{' doesn't exist after 'operator'. [+%d %s].\n", lineno, curfile); /* } */
+				warning("'{' doesn't exist after 'operator'. [+%d %s].", lineno, curfile); /* } */
 			break;
 		/* control statement check */
 		case CPP_THROW:
@@ -339,40 +374,44 @@ Cpp()
 		case CPP_TRY:
 		case CPP_WHILE:
 			if (wflag && !startmacro && level == 0)
-				fprintf(stderr, "Warning: Out of function. %8s [+%d %s]\n", token, lineno, curfile);
+				warning("Out of function. %8s [+%d %s]", token, lineno, curfile);
 			break;
 		case CPP_TYPEDEF:
 			if (tflag) {
-				char	savetok[MAXTOKEN];
-				int	savelineno = 0;
-				int	typedef_savelevel = level;
+				char savetok[MAXTOKEN];
+				int savelineno = 0;
+				int typedef_savelevel = level;
 
 				savetok[0] = 0;
-				c = nexttoken("{}(),;", reserved);
+
+				/* skip CV qualifiers */
+				do {
+					c = nexttoken("{}(),;", reserved_word);
+				} while (IS_CV_QUALIFIER(c) || c == '\n');
+
 				if (wflag && c == EOF) {
-					fprintf(stderr, "Warning: unexpected eof. [+%d %s]\n", lineno, curfile);
+					warning("unexpected eof. [+%d %s]", lineno, curfile);
 					break;
 				} else if (c == CPP_ENUM || c == CPP_STRUCT || c == CPP_UNION) {
 					char *interest_enum = "{},;";
 					int c_ = c;
 
-					c = nexttoken(interest_enum, reserved);
+					c = nexttoken(interest_enum, reserved_word);
 					/* read enum name if exist */
 					if (c == SYMBOL) {
 						if (target == SYM)
 							PUT(token, lineno, sp);
-						c = nexttoken(interest_enum, reserved);
+						c = nexttoken(interest_enum, reserved_word);
 					}
-					for (; c != EOF; c = nexttoken(interest_enum, reserved)) {
+					for (; c != EOF; c = nexttoken(interest_enum, reserved_word)) {
 						switch (c) {
-						case CP_IFDEF:
-						case CP_IFNDEF:
-						case CP_IF:
-						case CP_ELIF:
-						case CP_ELSE:
-						case CP_ENDIF:
-						case CP_UNDEF:
-							condition_macro(c);
+						case SHARP_IFDEF:
+						case SHARP_IFNDEF:
+						case SHARP_IF:
+						case SHARP_ELIF:
+						case SHARP_ELSE:
+						case SHARP_ENDIF:
+							condition_macro(c, target);
 							continue;
 						default:
 							break;
@@ -410,11 +449,7 @@ Cpp()
 					if (c == ';')
 						break;
 					if (wflag && c == EOF) {
-						fprintf(stderr, "Warning: unexpected eof. [+%d %s]\n", lineno, curfile);
-						break;
-					}
-					if (wflag && level != typedef_savelevel || c != '}') {
-						fprintf(stderr, "Warning: uneven {}. [+%d %s]\n", lineno, curfile);
+						warning("unexpected eof. [+%d %s]", lineno, curfile);
 						break;
 					}
 				} else if (c == SYMBOL) {
@@ -424,16 +459,15 @@ Cpp()
 						PUT(token, lineno, sp);
 				}
 				savetok[0] = 0;
-				while ((c = nexttoken("(),;", reserved)) != EOF) {
+				while ((c = nexttoken("(),;", reserved_word)) != EOF) {
 					switch (c) {
-					case CP_IFDEF:
-					case CP_IFNDEF:
-					case CP_IF:
-					case CP_ELIF:
-					case CP_ELSE:
-					case CP_ENDIF:
-					case CP_UNDEF:
-						condition_macro(c);
+					case SHARP_IFDEF:
+					case SHARP_IFNDEF:
+					case SHARP_IF:
+					case SHARP_ELIF:
+					case SHARP_ELSE:
+					case SHARP_ENDIF:
+						condition_macro(c, target);
 						continue;
 					default:
 						break;
@@ -443,7 +477,7 @@ Cpp()
 					else if (c == ')')
 						level--;
 					else if (c == SYMBOL) {
-						if (level > 0) {
+						if (level > typedef_savelevel) {
 							if (target == SYM)
 								PUT(token, lineno, sp);
 						} else {
@@ -463,17 +497,20 @@ Cpp()
 							savetok[0] = 0;
 						}
 					}
-					if (level == 0 && c == ';')
+					if (level == typedef_savelevel && c == ';')
 						break;
 				}
 				if (wflag) {
 					if (c == EOF)
-						fprintf(stderr, "Warning: unexpected eof. [+%d %s]\n", lineno, curfile);
-					else if (level > 0)
-						fprintf(stderr, "Warning: () block unmatched. (last at level %d.)[+%d %s]\n", level, lineno, curfile);
+						warning("unexpected eof. [+%d %s]", lineno, curfile);
+					else if (level != typedef_savelevel)
+						warning("() block unmatched. (last at level %d.)[+%d %s]", level, lineno, curfile);
 				}
 			}
 			break;
+		case CPP___ATTRIBUTE__:
+			process_attribute(target);
+			break;
 		default:
 			break;
 		}
@@ -481,9 +518,42 @@ Cpp()
 	strbuf_close(sb);
 	if (wflag) {
 		if (level != 0)
-			fprintf(stderr, "Warning: {} block unmatched. (last at level %d.)[+%d %s]\n", level, lineno, curfile);
+			warning("{} block unmatched. (last at level %d.)[+%d %s]", level, lineno, curfile);
 		if (piflevel != 0)
-			fprintf(stderr, "Warning: #if block unmatched. (last at level %d.)[+%d %s]\n", piflevel, lineno, curfile);
+			warning("#if block unmatched. (last at level %d.)[+%d %s]", piflevel, lineno, curfile);
+	}
+}
+/*
+ * process_attribute: skip attributes in __attribute__((...)).
+ *
+ *	r)	target type
+ */
+static void
+process_attribute(target)
+	int target;
+{
+	int brace = 0;
+	int c;
+	/*
+	 * Skip '...' in __attribute__((...))
+	 * but pick up symbols in it.
+	 */
+	while ((c = nexttoken("()", reserved_word)) != EOF) {
+		if (c == '(')
+			brace++;
+		else if (c == ')')
+			brace--;
+		else if (c == SYMBOL) {
+			if (target == REF) {
+				if (defined(token))
+					PUT(token, lineno, sp);
+			} else if (target == SYM) {
+				if (!defined(token))
+					PUT(token, lineno, sp);
+			}
+		}
+		if (brace == 0)
+			break;
 	}
 }
 /*
@@ -493,22 +563,21 @@ Cpp()
  */
 static int
 function_definition(target)
-int	target;
+	int target;
 {
-	int	c;
-	int     brace_level, isdefine;
+	int c;
+	int brace_level;
 
-	brace_level = isdefine = 0;
-	while ((c = nexttoken("()", reserved)) != EOF) {
+	brace_level = 0;
+	while ((c = nexttoken("()", reserved_word)) != EOF) {
 		switch (c) {
-		case CP_IFDEF:
-		case CP_IFNDEF:
-		case CP_IF:
-		case CP_ELIF:
-		case CP_ELSE:
-		case CP_ENDIF:
-		case CP_UNDEF:
-			condition_macro(c);
+		case SHARP_IFDEF:
+		case SHARP_IFNDEF:
+		case SHARP_IF:
+		case SHARP_ELIF:
+		case SHARP_ELSE:
+		case SHARP_ENDIF:
+			condition_macro(c, target);
 			continue;
 		default:
 			break;
@@ -522,10 +591,12 @@ int	target;
 		/* pick up symbol */
 		if (c == SYMBOL) {
 			if (target == REF) {
-				if (seems_datatype(token))
+				if (seems_datatype(token) && defined(token))
 					PUT(token, lineno, sp);
-			} else if (target == SYM)
-				PUT(token, lineno, sp);
+			} else if (target == SYM) {
+				if (!seems_datatype(token) || !defined(token))
+					PUT(token, lineno, sp);
+			}
 		}
 	}
 	if (c == EOF)
@@ -535,16 +606,18 @@ int	target;
 		return 0;
 	}
 	brace_level = 0;
-	while ((c = nexttoken(",;[](){}=", reserved)) != EOF) {
+	while ((c = nexttoken(",;[](){}=", reserved_word)) != EOF) {
 		switch (c) {
-		case CP_IFDEF:
-		case CP_IFNDEF:
-		case CP_IF:
-		case CP_ELIF:
-		case CP_ELSE:
-		case CP_ENDIF:
-		case CP_UNDEF:
-			condition_macro(c);
+		case SHARP_IFDEF:
+		case SHARP_IFNDEF:
+		case SHARP_IF:
+		case SHARP_ELIF:
+		case SHARP_ELSE:
+		case SHARP_ENDIF:
+			condition_macro(c, target);
+			continue;
+		case CPP___ATTRIBUTE__:
+			process_attribute(target);
 			continue;
 		default:
 			break;
@@ -553,12 +626,9 @@ int	target;
 			brace_level++;
 		else if (c == /* ( */')' || c == ']')
 			brace_level--;
-		else if (brace_level == 0 && (c == SYMBOL || IS_RESERVED(c)))
-			isdefine = 1;
-		else if (c == ';' || c == ',') {
-			if (!isdefine)
-				break;
-		} else if (c == '{' /* } */) {
+		else if (brace_level == 0 && (c == ';' || c == ','))
+			break;
+		else if (c == '{' /* } */) {
 			pushbacktoken();
 			return 1;
 		} else if (c == /* { */'}')
@@ -568,10 +638,12 @@ int	target;
 		/* pick up symbol */
 		if (c == SYMBOL) {
 			if (target == REF) {
-				if (seems_datatype(token))
+				if (seems_datatype(token) && defined(token))
 					PUT(token, lineno, sp);
-			} else if (target == SYM)
-				PUT(token, lineno, sp);
+			} else if (target == SYM) {
+				if (!seems_datatype(token) || !defined(token))
+					PUT(token, lineno, sp);
+			}
 		}
 	}
 	return 0;
@@ -581,13 +653,15 @@ int	target;
  * condition_macro: 
  *
  *	i)	cc	token
+ *	i)	target	current target
  */
 static void
-condition_macro(cc)
-	int	cc;
+condition_macro(cc, target)
+	int cc;
+	int target;
 {
 	cur = &pifstack[piflevel];
-	if (cc == CP_IFDEF || cc == CP_IFNDEF || cc == CP_IF) {
+	if (cc == SHARP_IFDEF || cc == SHARP_IFNDEF || cc == SHARP_IF) {
 		DBG_PRINT(piflevel, "#if");
 		if (++piflevel >= MAXPIFSTACK)
 			die("#if pifstack over flow. [%s]", curfile);
@@ -597,20 +671,20 @@ condition_macro(cc)
 		cur->if0only = 0;
 		if (peekc(0) == '0')
 			cur->if0only = 1;
-		else if ((cc = nexttoken(NULL, reserved)) == SYMBOL && !strcmp(token, "notdef"))
+		else if ((cc = nexttoken(NULL, reserved_word)) == SYMBOL && !strcmp(token, "notdef"))
 			cur->if0only = 1;
 		else
 			pushbacktoken();
-	} else if (cc == CP_ELIF || cc == CP_ELSE) {
+	} else if (cc == SHARP_ELIF || cc == SHARP_ELSE) {
 		DBG_PRINT(piflevel - 1, "#else");
 		if (cur->end == -1)
 			cur->end = level;
 		else if (cur->end != level && wflag)
-			fprintf(stderr, "Warning: uneven level. [+%d %s]\n", lineno, curfile);
+			warning("uneven level. [+%d %s]", lineno, curfile);
 		level = cur->start;
 		cur->if0only = 0;
-	} else if (cc == CP_ENDIF) {
-		int	minus = 0;
+	} else if (cc == SHARP_ENDIF) {
+		int minus = 0;
 
 		--piflevel;
 		if (piflevel < 0) {
@@ -619,17 +693,29 @@ condition_macro(cc)
 		}
 		DBG_PRINT(piflevel, "#endif");
 		if (minus) {
-			fprintf(stderr, "Warning: #if block unmatched. reseted. [+%d %s]\n", lineno, curfile);
+			if (wflag)
+				warning("#if block unmatched. reseted. [+%d %s]", lineno, curfile);
 		} else {
 			if (cur->if0only)
 				level = cur->start;
 			else if (cur->end != -1) {
 				if (cur->end != level && wflag)
-					fprintf(stderr, "Warning: uneven level. [+%d %s]\n", lineno, curfile);
+					warning("uneven level. [+%d %s]", lineno, curfile);
 				level = cur->end;
 			}
 		}
 	}
+	while ((cc = nexttoken(NULL, reserved_word)) != EOF && cc != '\n') {
+                if (cc == SYMBOL && strcmp(token, "defined") != 0) {
+			if (target == REF) {
+				if (dflag && defined(token))
+		                        PUT(token, lineno, sp);
+			} else if (target == SYM) {
+				if (!dflag || !defined(token))
+		                	PUT(token, lineno, sp);
+			}
+		}
+	}
 }
 /*
  * seems_datatype: decide whether or not it is a data type.
@@ -639,107 +725,17 @@ condition_macro(cc)
  */
 static int
 seems_datatype(token)
-const char *token;
+	const char *token;
 {
 	int length = strlen(token);
 	const char *p = token + length;
 
-	if (length < 3)
-		return 0;
-	if (strcmp(p - 2, "_t"))
+	if (length > 2 && strcmp(p - 2, "_t") == 0)
 		return 1;
 	for (p = token; *p; p++)
 		if (islower(*p))
 			return 0;
 	return 1;
-}
-		/* sorted by alphabet */
-static struct words words[] = {
-	{"##",		CP_SHARP},
-	{"#assert",	CP_ASSERT},
-	{"#define",	CP_DEFINE},
-	{"#elif",	CP_ELIF},
-	{"#else",	CP_ELSE},
-	{"#endif",	CP_ENDIF},
-	{"#error",	CP_ERROR},
-	{"#ident",	CP_IDENT},
-	{"#if",		CP_IF},
-	{"#ifdef",	CP_IFDEF},
-	{"#ifndef",	CP_IFNDEF},
-	{"#include",	CP_INCLUDE},
-	{"#line",	CP_LINE},
-	{"#pragma",	CP_PRAGMA},
-	{"#undef",	CP_UNDEF},
-	{"#warning",	CP_WARNING},
-	{"::",		CPP_SEP},
-	{"__P",		CPP___P},
-	{"__attribute__",CPP___ATTRIBUTE__},
-	{"asm",		CPP_ASM},
-	{"auto",	CPP_AUTO},
-	{"break",	CPP_BREAK},
-	{"case",	CPP_CASE},
-	{"catch",	CPP_CATCH},
-	{"char",	CPP_CHAR},
-	{"class",	CPP_CLASS},
-	{"const",	CPP_CONST},
-	{"continue",	CPP_CONTINUE},
-	{"default",	CPP_DEFAULT},
-	{"delete",	CPP_DELETE},
-	{"do",		CPP_DO},
-	{"double",	CPP_DOUBLE},
-	{"else",	CPP_ELSE},
-	{"enum",	CPP_ENUM},
-	{"explicit",	CPP_EXPLICIT},
-	{"extern",	CPP_EXTERN},
-	{"float",	CPP_FLOAT},
-	{"for",		CPP_FOR},
-	{"friend",	CPP_FRIEND},
-	{"goto",	CPP_GOTO},
-	{"if",		CPP_IF},
-	{"inline",	CPP_INLINE},
-	{"int",		CPP_INT},
-	{"long",	CPP_LONG},
-	{"mutable",	CPP_MUTABLE},
-	{"namespace",	CPP_NAMESPACE},
-	{"new",		CPP_NEW},
-	{"operator",	CPP_OPERATOR},
-	{"overload",	CPP_OVERLOAD},
-	{"private",	CPP_PRIVATE},
-	{"protected",	CPP_PROTECTED},
-	{"public",	CPP_PUBLIC},
-	{"register",	CPP_REGISTER},
-	{"return",	CPP_RETURN},
-	{"short",	CPP_SHORT},
-	{"signed",	CPP_SIGNED},
-	{"sizeof",	CPP_SIZEOF},
-	{"static",	CPP_STATIC},
-	{"struct",	CPP_STRUCT},
-	{"switch",	CPP_SWITCH},
-	{"template",	CPP_TEMPLATE},
-	{"this",	CPP_THIS},
-	{"throw",	CPP_THROW},
-	{"try",		CPP_TRY},
-	{"typedef",	CPP_TYPEDEF},
-	{"typename",	CPP_TYPENAME},
-	{"union",	CPP_UNION},
-	{"unsigned",	CPP_UNSIGNED},
-	{"using",	CPP_USING},
-	{"virtual",	CPP_VIRTUAL},
-	{"void",	CPP_VOID},
-	{"volatile",	CPP_VOLATILE},
-	{"while",	CPP_WHILE},
-};
-
-static int
-reserved(word)
-        char *word;
-{
-	struct words tmp;
-	struct words *result;
-
-	tmp.name = word;
-	result = (struct words *)bsearch(&tmp, words, sizeof(words)/sizeof(struct words), sizeof(struct words), cmp);
-	return (result != NULL) ? result->val : SYMBOL;
 }
 /*
  * whether or not C++.
@@ -752,7 +748,7 @@ isCpp()
 	cmode = 1;			/* allow token like '#xxx' */
 	cppmode = 1;			/* treat '::' as a token */
 
-	while ((cc = nexttoken(NULL, reserved)) != EOF) {
+	while ((cc = nexttoken(NULL, reserved_word)) != EOF) {
 		if (cc == CPP_CLASS || cc == CPP_TEMPLATE ||
 			cc == CPP_OPERATOR || cc == CPP_VIRTUAL) {
 			Cpp = 1;
