@@ -1,8 +1,6 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1998, 1999 Shigio Yamaguchi
+ * Copyright (c) 1999, 2000, 2001, 2002, 2003 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -43,49 +41,69 @@
 
 #include "global.h"
 #include "gctags.h"
+#include "const.h"
 
-const char *progname = "gctags";	/* program name */
-char	*notfunction;
-
-int	main(int, char **);
-static	void usage(void);
+int main(int, char **);
+static void usage(void);
+static void help(void);
 
+struct words {
+        const char *name;
+        int val;
+};
 struct words *words;
 static int tablesize;
 
-int	bflag;			/* -b: force level 1 block start */
-int	dflag;			/* -d: treat #define with no argument */
-int	eflag;			/* -e: force level 1 block end */
-int	nflag;			/* -n: doen't print tag */
-int	rflag;			/* -r: function reference */
-int	sflag;			/* -s: collect symbols */
-int	tflag;			/* -t: treat typedefs, structs, unions, and enums. */
-int	wflag;			/* -w: warning message */
-int	show_version;
-int	show_help;
-int	debug;
+/*
+ * language map.
+ *
+ * By default, default_map is used.
+ */
+static const char *default_map = "c:.c.h,yacc:.y,asm:.s.S,java:.java,cpp:.c++.cc.cpp.cxx.hxx.hpp.C.H,php:.php.php3.phtml";
+static char *langmap;
 
-int	yaccfile;		/* yacc file */
+int bflag;			/* -b: force level 1 block start */
+int dflag;			/* -d: treat #define with no argument */
+int eflag;			/* -e: force level 1 block end */
+int nflag;			/* -n: doen't print tag */
+int rflag;			/* -r: function reference */
+int sflag;			/* -s: collect symbols */
+int tflag;			/* -t: treat typedefs, structs, unions, and enums. */
+int wflag;			/* -w: warning message */
+int vflag;			/* -v: verbose mode */
+int do_check;
+int show_version;
+int show_help;
+int debug;
 
 static void
 usage()
 {
-	(void)fprintf(stderr, "usage: gctags [-bdenrstw] file ...\n");
+	fputs(usage_const, stderr);
 	exit(2);
 }
-
+static void
+help()
+{
+	fputs(usage_const, stdout);
+	fputs(help_const, stdout);
+	exit(0);
+}
 static struct option const long_options[] = {
 	{"begin-block", no_argument, NULL, 'b'},
+	{"check", no_argument, &do_check, 1},
 	{"define", no_argument, NULL, 'd'},
 	{"end-block", no_argument, NULL, 'e'},
 	{"no-tags", no_argument, NULL, 'n'},
 	{"reference", no_argument, NULL, 'r'},
 	{"symbol", no_argument, NULL, 's'},
 	{"typedef", no_argument, NULL, 't'},
+	{"verbose", no_argument, NULL, 'v'},
 	{"warning", no_argument, NULL, 'w'},
 
 	/* long name only */
 	{"debug", no_argument, &debug, 1},
+	{"langmap", required_argument, NULL, 0},
 	{"version", no_argument, &show_version, 1},
 	{"help", no_argument, &show_help, 1},
 	{ 0 }
@@ -93,15 +111,19 @@ static struct option const long_options[] = {
 
 int
 main(argc, argv)
-	int	argc;
-	char	**argv;
+	int argc;
+	char **argv;
 {
-	int	optchar;
-	char	*p;
+	char *p;
+	int optchar;
+	int option_index = 0;
 
-	while ((optchar = getopt_long(argc, argv, "bdenrstw", long_options, NULL)) != EOF) {
+	while ((optchar = getopt_long(argc, argv, "bdenrstvw", long_options, &option_index)) != EOF) {
 		switch(optchar) {
 		case 0:
+			p = (char *)long_options[option_index].name;
+			if (!strcmp(p, "langmap"))
+				langmap = optarg;	
 			break;
 		case 'b':
 			bflag++;
@@ -126,6 +148,9 @@ main(argc, argv)
 		case 't':
 			tflag++;
 			break;
+		case 'v':
+			vflag++;
+			break;
 		case 'w':
 			wflag++;
 			break;
@@ -135,9 +160,14 @@ main(argc, argv)
 		}
 	}
 	if (show_version)
-		version(NULL, 0);
+		version(NULL, vflag);
 	else if (show_help)
-		usage();
+		help();
+	else if (do_check) {
+		fprintf(stdout, "Part of GLOBAL\n");
+		exit(0);
+	}
+	setup_langmap(langmap ? langmap : default_map);
 
         argc -= optind;
         argv += optind;
@@ -146,14 +176,26 @@ main(argc, argv)
 		usage();
 	if (getenv("GTAGSWARNING"))
 		wflag++;	
-	if (test("r", NOTFUNCTION)) {
-		FILE	*ip;
-		STRBUF	*sb = strbuf_open(0);
-		STRBUF	*ib = strbuf_open(0);
-		char	*p;
-		int	i;
+	/*
+	 * This is a hack for FreeBSD.
+	 * In the near future, it will be removed.
+	 */
+#ifdef __DJGPP__
+	if (test("r", NOTFUNCTION) || test("r", DOS_NOTFUNCTION))
+#else
+	if (test("r", NOTFUNCTION))
+#endif
+	{
+		FILE *ip;
+		STRBUF *sb = strbuf_open(0);
+		STRBUF *ib = strbuf_open(0);
+		char *p;
+		int i;
 
 		if ((ip = fopen(NOTFUNCTION, "r")) == 0)
+#ifdef __DJGPP__
+			if ((ip = fopen(DOS_NOTFUNCTION, "r")) == 0)
+#endif
 			die("'%s' cannot read.", NOTFUNCTION);
 		for (tablesize = 0; (p = strbuf_fgets(ib, ip, STRBUF_NOCRLF)) != NULL; tablesize++)
 			strbuf_puts0(sb, p);
@@ -164,6 +206,8 @@ main(argc, argv)
 		 * Don't free *p.
 		 */
 		p = (char *)malloc(strbuf_getlen(sb) + 1);
+		if (p == NULL)
+			die("short of memory.");
 		memcpy(p, strbuf_value(sb), strbuf_getlen(sb) + 1);
 		for (i = 0; i < tablesize; i++) {
 			words[i].name = p;
@@ -173,55 +217,55 @@ main(argc, argv)
 		strbuf_close(sb);
 		strbuf_close(ib);
 	}
+
+	/*
+	 * pick up files and parse them.
+	 */
 	for (; argc > 0; argv++, argc--) {
-		if (!opentoken(argv[0]))
-			die("'%s' cannot open.", argv[0]);
-#ifdef _WIN32
+		const char *lang, *suffix;
+
+#if defined(_WIN32) || defined(__DJGPP__)
 		/* Lower case the file name since names are case insensitive */
 		strlwr(argv[0]);
 #endif
+		/* get suffix of the path. */
+		suffix = locatestring(argv[0], ".", MATCH_LAST);
+		if (!suffix)
+			continue;
+		lang = decide_lang(suffix);
+		if (lang == NULL)
+			continue;
+
 		/*
-		 * yacc
+		 * Initialize token parser. Php() and assembler() use different parser.
 		 */
-		if (locatestring(argv[0], ".y", MATCH_AT_LAST))
-			C(YACC);
+		if (strcmp(lang, "php") && strcmp(lang, "asm"))
+			if (!opentoken(argv[0]))
+				die("'%s' cannot open.", argv[0]);
+
+		if (!strcmp(suffix, ".h") && isCpp())
+			lang = "cpp";
+		if (vflag)
+			fprintf(stderr, "suffix '%s' assumed language '%s'.\n", suffix, lang);
+
 		/*
-		 * assembler
+		 * call language specific parser.
 		 */
-		else if (locatestring(argv[0], ".s", MATCH_AT_LAST)	||
-			locatestring(argv[0], ".S", MATCH_AT_LAST))
-			assembler();
-		/*
-		 * java
-		 */
-		else if (locatestring(argv[0], ".java", MATCH_AT_LAST))
-			java();
-		/*
-		 * C++
-		 */
-		else if (locatestring(argv[0], ".c++", MATCH_AT_LAST)	||
-			locatestring(argv[0], ".cc", MATCH_AT_LAST)	||
-			locatestring(argv[0], ".cpp", MATCH_AT_LAST)	||
-			locatestring(argv[0], ".cxx", MATCH_AT_LAST)	||
-			locatestring(argv[0], ".hxx", MATCH_AT_LAST)	||
-			locatestring(argv[0], ".C", MATCH_AT_LAST)	||
-			locatestring(argv[0], ".H", MATCH_AT_LAST))
-			Cpp();
-		/*
-		 * C
-		 */
-		else if (locatestring(argv[0], ".c", MATCH_AT_LAST))
+		if (!strcmp(lang, "c")) {
 			C(0);
-		/*
-		 * C or C++
-		 */
-		else if (locatestring(argv[0], ".h", MATCH_AT_LAST)) {
-			if (isCpp())
-				Cpp();
-			else
-				C(0);
+		} else if (!strcmp(lang, "yacc")) {
+			C(YACC);
+		} else if (!strcmp(lang, "asm")) {
+			assembler(argv[0]);
+		} else if (!strcmp(lang, "java")) {
+			java();
+		} else if (!strcmp(lang, "cpp")) {
+			Cpp();
+		} else if (!strcmp(lang, "php")) {
+			php(argv[0]);
 		}
-		closetoken();
+		if (strcmp(lang, "php") && strcmp(lang, "asm"))
+			closetoken();
 	}
 	return 0;
 }
@@ -235,7 +279,7 @@ cmp(s1, s2)
 
 int
 isnotfunction(name)
-	char	*name;
+	char *name;
 {
 	struct words tmp;
 	struct words *result;
