@@ -40,8 +40,9 @@ extern int nflag;
 extern int rflag;
 extern int sflag;
 extern int tflag;
+extern int vflag;
 extern int wflag;
-extern int yaccfile;
+extern int debug;
 
 #define PUT(tag, lno, line) do {					\
 	DBG_PRINT(level, line);						\
@@ -57,6 +58,6 @@ int cmp(const void *, const void *);
 void C(int);
 void Cpp(void);
 int isCpp(void);
-void assembler(void);
+void assembler(const char *);
 void java(void);
 void php(const char *);
