@@ -483,9 +483,6 @@ delete_stack(sp)
 /*----------------------------------------------------------------------*/
 /* Main procedure							*/
 /*----------------------------------------------------------------------*/
-#define KEYWORD		"include"
-#define KEYWORD_LENGTH	7
-
 /*
  * extract_lastname: extract the last name of include line.
  *
@@ -508,7 +505,7 @@ extract_lastname(image, is_php)
 	 * PHP: include('xxx/yyy/zzz');
 	 */
 	p = image;
-	while (*p && isspace(*p))		/* skip space */
+	while (*p && isspace((unsigned char)*p))		/* skip space */
 		p++;
 	if (!*p)
 		return NULL;
@@ -516,16 +513,37 @@ extract_lastname(image, is_php)
 		if (is_php)
 			return NULL;
 		p++;
-		while (*p && isspace(*p))	/* skip space */
+		while (*p && isspace((unsigned char)*p))	/* skip space */
 			p++;
 		if (!*p)
 			return NULL;
 	}
-	if (strncmp(p, KEYWORD, KEYWORD_LENGTH))
-		return NULL;
-	p += KEYWORD_LENGTH;
-	while (*p && isspace(*p))		/* skip space */
+	/*
+	 * If match to one of the include keywords then points
+	 * the following character of the keyword.
+	 *            p
+	 *            v
+	 * ... include ....
+	 */
+	if (is_php) {
+		if ((p = locatestring(p, "include", MATCH_AT_FIRST)) == NULL)
+			return NULL;
+	} else {
+		char *q;
+
+		if (((q = locatestring(p, "include_next", MATCH_AT_FIRST)) == NULL) &&
+		    ((q = locatestring(p, "import", MATCH_AT_FIRST)) == NULL) &&
+		    ((q = locatestring(p, "include", MATCH_AT_FIRST)) == NULL))
+			return NULL;
+		p = q;
+	}
+	while (*p && isspace((unsigned char)*p))		/* skip space */
 		p++;
+	if (is_php && *p == '(') {
+		p++;
+		while (*p && isspace((unsigned char)*p))	/* skip space */
+			p++;
+	}
 	sep = *p;
 	if (is_php) {
 		if (sep != '\'' && sep != '"')
@@ -564,7 +582,7 @@ makefileindex(file, files)
 	char *file;
 	STRBUF *files;
 {
-	FILE *FIND, *FILEMAP, *PIPE, *FILES, *STDOUT, *op = NULL;
+	FILE *FIND, *FILEMAP, *FILES, *STDOUT, *op = NULL;
 	char *_;
 	int count = 0;
 	char indexlink[80];
@@ -574,26 +592,35 @@ makefileindex(file, files)
 	char *target = (Fflag) ? "mains" : "_top";
 	struct dirstack *dirstack = make_stack("dirstack");
 	struct dirstack *fdstack = make_stack("fdstack");
-	char *command = (other_files) ? "gtags --find --other | sort -t / -k 2"
+	char *command = (other_files) ? "gtags --find --other | gnusort -t / -k 2"
 					: "gtags --find";
 	struct dirstack *push = make_stack("push");
 	struct dirstack *pop = make_stack("pop");
 	/*
 	 * for collecting include files.
 	 */
-	struct data *inc;
 	int flags = REG_EXTENDED;
-	char *pattern;
-	regex_t is_include_file, is_c_file;
+	regex_t is_include_file;
 
 	if (w32)
 		flags |= REG_ICASE;
-	pattern = "\\.(h|hxx|hpp|H|inc\\.php)$";
-	if (regcomp(&is_include_file, pattern, flags) != 0)
-		die("cannot compile regular expression '%s'.", pattern);
-	pattern = "\\.(h|c|y|c\\+\\+|cc|cpp|cxx|hxx|hpp|C|H)$";
-	if (regcomp(&is_c_file, pattern, flags) != 0)
-		die("cannot compile regular expression '%s'.", pattern);
+	strbuf_reset(sb);
+	strbuf_puts(sb, "\\.(");
+	{
+		char *p = include_file_suffixes;
+		int c;
+
+		while ((c = (unsigned char)*p++) != '\0') {
+			if (isregexchar(c))
+				strbuf_putc(sb, '\\');
+			else if (c == ',')
+				c = '|';
+			strbuf_putc(sb, c);
+		}
+	}
+	strbuf_puts(sb, ")$");
+	if (regcomp(&is_include_file, strbuf_value(sb), flags) != 0)
+		die("cannot compile regular expression '%s'.", strbuf_value(sb));
 
 	/*
 	 * preparations.
@@ -767,7 +794,15 @@ makefileindex(file, files)
 		}
 		strbuf_sprintf(sb, " TARGET=%s TITLE='%s'>", target, _);
 		if (icon_list) {
-			char *text_icon = regexec(&is_c_file, _, 0, 0, 0) == 0 ? c_icon : file_icon;
+			const char *lang, *suffix, *text_icon;
+
+			if ((suffix = locatestring(_, ".", MATCH_LAST)) != NULL
+			    && (lang = decide_lang(suffix)) != NULL
+			    && (strcmp(lang, "c") == 0 || strcmp(lang, "cpp") == 0
+			       || strcmp(lang, "yacc") == 0))
+				text_icon = c_icon;
+			else
+				text_icon = file_icon;
 			strbuf_puts(sb, "<IMG SRC=");
 			if (count_stack(dirstack))
 				strbuf_puts(sb, "../");
@@ -830,13 +865,32 @@ makefileindex(file, files)
 	fclose(FILES);
 	file_count++;
 
+	delete_stack(dirstack);
+	delete_stack(fdstack);
+	delete_stack(push);
+	delete_stack(pop);
+
+	strbuf_close(sb);
+	strbuf_close(input);
+	return count;
+}
+void
+makeincludeindex()
+{
+	FILE *PIPE;
+	STRBUF *input = strbuf_open(0);
+	char *command;
+	char *_;
+	struct data *inc;
+	char *target = (Fflag) ? "mains" : "_top";
+
 	/*
 	 * Pick up include pattern.
 	 *
 	 * C: #include "xxx.h"
 	 * PHP: include("xxx.inc.php");
 	 */
-	command = "global -gnx \"^[ \\t]*(#[ \\t]*include|include[ \\t]*\\()\"";
+	command = "global -gnx \"^[ \\t]*(#[ \\t]*(import|include)|include[ \\t]*\\()\"";
 	if ((PIPE = popen(command, "r")) == NULL)
 		die("cannot fork.");
 	strbuf_reset(input);
@@ -845,6 +899,7 @@ makefileindex(file, files)
                 char *nouse, *lno, *filename, *image;
 		char *last, buf[MAXBUFLEN];
 		int is_php = 0;
+		const char *lang, *suffix;
 
 		if (split(_, 4, &ptable) < 4) {
 			recover(&ptable);
@@ -854,8 +909,10 @@ makefileindex(file, files)
 		lno = ptable.part[1].start;
 		filename = ptable.part[2].start;
 		image = ptable.part[3].start;
-	
-		if (locatestring(image, ".php", MATCH_AT_LAST))
+
+		if ((suffix = locatestring(filename, ".", MATCH_LAST)) != NULL
+		    && (lang = decide_lang(suffix)) != NULL
+		    && strcmp(lang, "php") == 0)
 			is_php = 1;
 		last = extract_lastname(image, is_php);
 		if (last == NULL || get_inc(last) == NULL)
@@ -877,7 +934,8 @@ makefileindex(file, files)
 		}
 		put_included(last, buf);
 	}
-	pclose(PIPE);
+	if (pclose(PIPE) < 0)
+		die("terminated abnormally.");
 
 	for (inc = first_inc(); inc; inc = next_inc()) {
 		char *last = inc->name;
@@ -960,12 +1018,5 @@ makefileindex(file, files)
 			data->id = no;
 		}
 	}
-	delete_stack(dirstack);
-	delete_stack(fdstack);
-	delete_stack(push);
-	delete_stack(pop);
-
-	strbuf_close(sb);
 	strbuf_close(input);
-	return count;
 }
