@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1996, 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2001, 2002, 2003 Tama Communications Corporation
+ * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -36,11 +36,87 @@
 #include "path2url.h"
 #include "htags.h"
 
+/*----------------------------------------------------------------------*/
+/* Parser switch							*/
+/*----------------------------------------------------------------------*/
+/*
+ * This is the linkage section of each parsers.
+ * If you want to support new language, you must define two procedures:
+ *	1. Initializing procedure.
+ *		Called once first with an input file descripter.
+ *	2. Executing procedure.
+ *		Called repeatedly until returning EOF.
+ *		It should read from above descripter and write HTML
+ *		using output procedures in this module.
+ */
+struct lang_entry {
+	char *lang_name;
+	void (*init_proc)(FILE *);		/* initializing procedure */
+	int (*exec_proc)(void);			/* executing procedure */
+};
+
+/* initializing procedures */
+void c_parser_init(FILE *);
+void yacc_parser_init(FILE *);
+void cpp_parser_init(FILE *);
+void java_parser_init(FILE *);
+void php_parser_init(FILE *);
+void asm_parser_init(FILE *);
+
+/* executing procedures */
+int clex(void);
+int cpplex(void);
+int javalex(void);
+int phplex(void);
+int asmlex(void);
+
+/*
+ * The first entry is default language.
+ */
+struct lang_entry lang_switch[] = {
+	/* lang_name	init_proc 		exec_proc */
+	{"c",		c_parser_init,		clex},		/* DEFAULT */
+	{"yacc",	yacc_parser_init,	clex},
+	{"cpp",		cpp_parser_init,	cpplex},
+	{"java",	java_parser_init,	javalex},
+	{"php",		php_parser_init,	phplex},
+	{"asm",		asm_parser_init,	asmlex}
+};
+#define DEFAULT_ENTRY &lang_switch[0]
+
+/*
+ * get language entry.
+ *
+ *	i)	lang	language name (NULL means 'not specified'.)
+ *	r)		language entry
+ */
+static struct lang_entry *
+get_lang_entry(lang)
+	const char *lang;
+{
+	int i, size = sizeof(lang_switch) / sizeof(struct lang_entry);
+
+	/*
+	 * if language not specified, it assumes default language.
+	 */
+	if (lang == NULL)
+		return DEFAULT_ENTRY;
+	for (i = 0; i < size; i++)
+		if (!strcmp(lang, lang_switch[i].lang_name))
+			return &lang_switch[i];
+	/*
+	 * if specified language not found, it assumes default language.
+	 */
+	return DEFAULT_ENTRY;
+}
+/*----------------------------------------------------------------------*/
+/* Input/Output								*/
+/*----------------------------------------------------------------------*/
 /*
  * Input/Output descriptor.
  */
-FILE *out;
-FILE *in;
+static FILE *out;
+static FILE *in;
 
 static STRBUF *outbuf;
 static char *curpfile;
@@ -48,11 +124,14 @@ static int warned;
 static int last_lineno;
 
 /*
- * IO routine.
+ * Open source file.
+ *
+ *	i)	file	source file name
+ *	r)		file pointer
  */
 static FILE *
 open_input_file(file)
-char *file;
+	char *file;
 {
 	char command[MAXFILLEN];
 	FILE *ip;
@@ -65,12 +144,22 @@ char *file;
 	warned = 0;
 	return ip;
 }
+/*
+ * Close source file.
+ */
 static void
-close_input_file()
+close_input_file(ip)
+	FILE *ip;
 {
-	if (pclose(in) < 0)
+	if (pclose(ip) < 0)
 		die("command 'gtags --expand -%d' failed.", tabs);
 }
+/*
+ * Open HTML file.
+ *
+ *	i)	file	HTML file name
+ *	r)		file pointer
+ */
 static FILE *
 open_output_file(file)
 	char *file;
@@ -94,38 +183,45 @@ open_output_file(file)
 		strbuf_reset(outbuf);
 	return op;
 }
+/*
+ * Close HTML file.
+ */
 static void
-close_output_file()
+close_output_file(op)
+	FILE *op;
 {
 	if (cflag) {
-		if (pclose(out) < 0)
+		if (pclose(op) < 0)
 			die("command 'gzip -c' failed.");
 	} else
-		fclose(out);
+		fclose(op);
 }
 /*
- * reverse of atoi
+ * Put a character to HTML as is.
+ *
+ * You should use this function to put a control character.
  */
-static char *
-itoa(n)
-	int n;
-{
-	static char buf[32];
-
-	snprintf(buf, sizeof(buf), "%d", n);
-	return buf;
-}
 void
-echoc(int c)
+echoc(c)
+	int c;
 {
         strbuf_putc(outbuf, c);
 }
+/*
+ * Put a string to HTML as is.
+ *
+ * You should use this function to put a control sequence.
+ */
 void
-echos(const char *s)
+echos(s)
+	const char *s;
 {
         strbuf_puts(outbuf, s);
 }
 
+/*----------------------------------------------------------------------*/
+/* HTML output								*/
+/*----------------------------------------------------------------------*/
 /*
  * fill_anchor: fill anchor into file name
  *
@@ -407,7 +503,7 @@ put_include_anchor(inc, path)
 	strbuf_puts(outbuf, "</A>");
 }
 /*
- * Tag level output functions.
+ * Put a reserved word. (if, while, ...)
  */
 void
 put_reserved_word(word)
@@ -417,6 +513,9 @@ put_reserved_word(word)
 	strbuf_puts(outbuf, word);
 	strbuf_puts(outbuf, reserved_end);
 }
+/*
+ * Put a macro (#define,#undef,...) 
+ */
 void
 put_macro(word)
         char *word;
@@ -425,6 +524,59 @@ put_macro(word)
 	strbuf_puts(outbuf, word);
 	strbuf_puts(outbuf, sharp_end);
 }
+/*
+ * Print warning message when unkown preprocessing directive is found.
+ */
+void
+unknown_preprocessing_directive(word, lineno)
+	char *word;
+	int lineno;
+{
+	word = strtrim(word, TRIM_ALL, NULL);
+	warning("unknown preprocessing directive '%s'. [+%d %s]", word, lineno, curpfile);
+	if (colorize_warned_line)
+		warned = 1;
+}
+/*
+ * Print warning message when unexpected eof.
+ */
+void
+unexpected_eof(lineno)
+	int lineno;
+{
+	warning("unexpected eof. [+%d %s]", lineno, curpfile);
+	if (colorize_warned_line)
+		warned = 1;
+}
+/*
+ * Print warning message when unknown yacc directive is found.
+ */
+void
+unknown_yacc_directive(word, lineno)
+	char *word;
+	int lineno;
+{
+	warning("unknown yacc directive '%s'. [+%d %s]", word, lineno, curpfile);
+	if (colorize_warned_line)
+		warned = 1;
+}
+/*
+ * Print warning message when unmatched brace is found.
+ */
+void
+missing_left(word, lineno)
+	char *word;
+	int lineno;
+{
+	warning("missing left '%s'. [+%d %s]", word, lineno, curpfile);
+	if (colorize_warned_line)
+		warned = 1;
+}
+/*
+ * Put a character with HTML quoting.
+ *
+ * If you want to put '<', '>' and '&', you should echoc() instead.
+ */
 void
 put_char(c)
         int c;
@@ -438,6 +590,11 @@ put_char(c)
         else
 		strbuf_putc(outbuf, c);
 }
+/*
+ * Put a string with HTML quoting.
+ *
+ * If you want to put HTML tag itself, you should echoc() instead.
+ */
 void
 put_string(s)
         char *s;
@@ -445,6 +602,9 @@ put_string(s)
 	for (; *s; s++)
 		put_char(*s);
 }
+/*
+ * Put brace ('{', '}')
+ */
 void
 put_brace(text)
         char *text;
@@ -460,6 +620,9 @@ put_brace(text)
 static char lineno_format[32];
 static char *guide = NULL;
 
+/*
+ * Begin of line processing.
+ */
 void
 put_begin_of_line(lineno)
         int lineno;
@@ -476,6 +639,15 @@ put_begin_of_line(lineno)
                 guide = NULL;
         }
 }
+/*
+ * End of line processing.
+ *
+ *	i)	lineno	current line number
+ *	gi)	outbuf	HTML line image
+ *
+ * The outbuf(string buffer) has HTML image of the line.
+ * This function flush and clear it.
+ */
 void
 put_end_of_line(lineno)
 	int lineno;
@@ -517,11 +689,6 @@ put_end_of_line(lineno)
  *       i)      html  HTML file       - Write to
  *       i)      notsource 1: isn't source, 0: source.
  */
-void c_parser_init(FILE *);
-void cpp_parser_init(FILE *);
-void java_parser_init(FILE *);
-void php_parser_init(FILE *);
-
 void
 src2html(src, html, notsource)
 	char *src;
@@ -529,7 +696,6 @@ src2html(src, html, notsource)
 	int notsource;
 {
 	char indexlink[128];
-	int clex(void), cpplex(void), javalex(void), phplex(void);
 
 	/*
 	 * setup lineno format.
@@ -613,6 +779,7 @@ src2html(src, html, notsource)
 			fputc('\n', out);
 		}
 		fprintf(out, "%s\n", verbatim_end);
+		strbuf_close(sb);
         }
 	/*
 	 * It's source code.
@@ -626,7 +793,7 @@ src2html(src, html, notsource)
                 /*
                  * INCLUDED FROM index.
                  */
-		basename = locatestring(src, "/", MATCH_LAST);;
+		basename = locatestring(src, "/", MATCH_LAST);
 		if (basename)
 			basename++;
 		else
@@ -635,8 +802,11 @@ src2html(src, html, notsource)
 		if (incref) {
 			fputs("<H2><A HREF=", out);
 			if (incref->count > 1) {
+				char s_count[32];
+
+				snprintf(s_count, sizeof(s_count), "%d", incref->count);
 				fprintf(out, "../%s/%d.%s", INCREFS, incref->id, HTML);
-				fprintf(out, " TITLE='%s'>", tooltip('I', -1, itoa(incref->count)));
+				fprintf(out, " TITLE='%s'>", tooltip('I', -1, s_count));
 			} else {
 				char *lno, *filename, *save;
 				char *p = strbuf_value(incref->contents);
@@ -682,32 +852,31 @@ src2html(src, html, notsource)
 		/*
 		 * print source code
 		 */
-        	fprintf(out, "%s\n", verbatim_begin);
-		if (locatestring(src, ".java", MATCH_AT_LAST)) {
-			java_parser_init(in);
-			while (javalex())
-				;
-		} else if (locatestring(src, ".php", MATCH_AT_LAST) ||
-			locatestring(src, ".php3", MATCH_AT_LAST) ||
-			locatestring(src, ".phtml", MATCH_AT_LAST)) {
-			php_parser_init(in);
-			while (phplex())
-				;
-		} else if (locatestring(src, ".h", MATCH_AT_LAST) ||
-			locatestring(src, ".c++", MATCH_AT_LAST) ||
-			locatestring(src, ".cc", MATCH_AT_LAST) ||
-			locatestring(src, ".cpp", MATCH_AT_LAST) ||
-			locatestring(src, ".cxx", MATCH_AT_LAST) ||
-			locatestring(src, ".hxx", MATCH_AT_LAST) ||
-			locatestring(src, ".hpp", MATCH_AT_LAST) ||
-			locatestring(src, ".C", MATCH_AT_LAST) ||
-			locatestring(src, ".H", MATCH_AT_LAST)) {
-			cpp_parser_init(in);
-			while (cpplex())
-				;
-		} else {
-			c_parser_init(in);
-			while (clex())
+		fprintf(out, "%s\n", verbatim_begin);
+		{
+			const char *suffix = locatestring(src, ".", MATCH_LAST);
+			const char *lang = NULL;
+			struct lang_entry *ent;
+
+			/*
+			 * Decide language.
+			 */
+			if (suffix)
+				lang = decide_lang(suffix);
+			/*
+			 * Select parser.
+			 * If lang == NULL then default parser is selected.
+			 */
+			ent = get_lang_entry(lang);
+			/*
+			 * Initialize parser.
+			 */
+			ent->init_proc(in);
+			/*
+			 * Execute parser.
+			 * Exec_proc() is called repeatedly until returning EOF.
+			 */
+			while (ent->exec_proc())
 				;
 		}
         	fprintf(out, "%s\n", verbatim_end);
@@ -724,6 +893,6 @@ src2html(src, html, notsource)
 
 	if (!notsource)
 		anchor_unload();
-	close_output_file();
-	close_input_file();
+	close_output_file(out);
+	close_input_file(in);
 }
