@@ -1,8 +1,6 @@
 /*
- * Copyright (c) 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001, 2002
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1998, 1999 Shigio Yamaguchi
+ * Copyright (c) 1999, 2000, 2001, 2002 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -38,26 +36,28 @@
 #include "gparam.h"
 #include "conf.h"
 #include "die.h"
+#include "env.h"
 #include "locatestring.h"
 #include "makepath.h"
+#include "path.h"
 #include "strbuf.h"
 #include "strlimcpy.h"
 #include "strmake.h"
 #include "test.h"
 #include "usable.h"
 
-static FILE	*fp;
-static STRBUF	*ib;
-static char	*confline;
+static FILE *fp;
+static STRBUF *ib;
+static char *confline;
 /*
  * 8 level nested tc= or include= is allowed.
  */
-static int	allowed_nest_level = 8;
-static int	opened;
+static int allowed_nest_level = 8;
+static int opened;
 
-static void	trim(char *);
-static char	*readrecord(const char *);
-static void	includelabel(STRBUF *, const char *, int);
+static void trim(char *);
+static char *readrecord(const char *);
+static void includelabel(STRBUF *, const char *, int);
 
 #ifndef isblank
 #define isblank(c)	((c) == ' ' || (c) == '\t')
@@ -73,10 +73,10 @@ static void	includelabel(STRBUF *, const char *, int);
  */
 static void
 trim(l)
-char	*l;
+	char *l;
 {
-	char	*f, *b;
-	int	colon = 0;
+	char *f, *b;
+	int colon = 0;
 
 	/*
 	 * delete blanks.
@@ -111,13 +111,13 @@ char	*l;
  * o append following line.
  * o format check.
  */
-static char	*
+static char *
 readrecord(label)
-const char *label;
+	const char *label;
 {
-	char	*p;
-	int	flag = STRBUF_NOCRLF;
-	int	count = 0;
+	char *p;
+	int flag = STRBUF_NOCRLF;
+	int count = 0;
 
 	rewind(fp);
 	while ((p = strbuf_fgets(ib, fp, flag)) != NULL) {
@@ -142,7 +142,7 @@ const char *label;
 				die("invalid config file format (line %d).", count);
 			if (!strcmp(label, candidate)) {
 				if (!(p = locatestring(p, ":", MATCH_FIRST)))
-					die("invalid config file format (line %d).", strbuf_value(ib));
+					die("invalid config file format (line %d).", count);
 				p = strdup(p);
 				if (!p)
 					die("short of memory.");
@@ -174,11 +174,11 @@ const char *label;
  */
 static	void
 includelabel(sb, label, level)
-STRBUF	*sb;
-const char *label;
-int	level;
+	STRBUF	*sb;
+	const char *label;
+	int	level;
 {
-	char	*savep, *p, *q;
+	char *savep, *p, *q;
 
 	if (++level > allowed_nest_level)
 		die("nested include= (or tc=) over flow.");
@@ -214,8 +214,12 @@ configpath() {
 	/*
 	 * if GTAGSCONF not set then check standard config files.
 	 */
-	else if ((p = getenv("HOME")) && test("r", makepath(p, GTAGSRC, NULL)))
+	else if ((p = get_home_directory()) && test("r", makepath(p, GTAGSRC, NULL)))
 		strlimcpy(config, makepath(p, GTAGSRC, NULL), sizeof(config));
+#ifdef __DJGPP__
+	else if ((p = get_home_directory()) && test("r", makepath(p, DOS_GTAGSRC, NULL)))
+		strlimcpy(config, makepath(p, DOS_GTAGSRC, NULL), sizeof(config));
+#endif
 	else if (test("r", GTAGSCONF))
 		strlimcpy(config, GTAGSCONF, sizeof(config));
 	else if (test("r", OLD_GTAGSCONF))
@@ -254,9 +258,9 @@ openconf()
 			die("short of memory.");
 	}
 	/*
-	 * if it doesn't start with '/' then assumed config value itself.
+	 * if it is not an absolute path then assumed config value itself.
 	 */
-	else if (*config != '/') {
+	else if (!isabspath(config)) {
 		confline = strdup(config);
 		if (!confline)
 			die("short of memory.");
@@ -336,10 +340,6 @@ openconf()
 		strbuf_puts(sb, path);
 		strbuf_puts(sb, " -dts %s");
 	}
-	if (!getconfs("sort_command", NULL))
-		strbuf_puts(sb, ":sort_command=sort");
-	if (!getconfs("sed_command", NULL))
-		strbuf_puts(sb, ":sed_command=sed");
 	strbuf_unputc(sb, ':');
 	strbuf_putc(sb, ':');
 	confline = strdup(strbuf_value(sb));
@@ -358,11 +358,11 @@ openconf()
  */
 int
 getconfn(name, num)
-const char *name;
-int	*num;
+	const char *name;
+	int *num;
 {
-	char	*p;
-	char	buf[MAXPROPLEN+1];
+	char *p;
+	char buf[MAXPROPLEN+1];
 
 	if (!opened)
 		openconf();
@@ -384,13 +384,13 @@ int	*num;
  */
 int
 getconfs(name, sb)
-const char *name;
-STRBUF	*sb;
+	const char *name;
+	STRBUF *sb;
 {
-	char	*p;
-	char	buf[MAXPROPLEN+1];
-	int	all = 0;
-	int	exist = 0;
+	char *p;
+	char buf[MAXPROPLEN+1];
+	int all = 0;
+	int exist = 0;
 
 	if (!opened)
 		openconf();
@@ -403,7 +403,7 @@ STRBUF	*sb;
 			strbuf_putc(sb, ',');		
 		exist = 1;
 		for (p += strlen(buf); *p && *p != ':'; p++) {
-			if (*p == '\\')	/* quoted charactor */
+			if (*p == '\\')	/* quoted character */
 				p++;
 			if (sb)
 				strbuf_putc(sb, *p);
@@ -411,6 +411,19 @@ STRBUF	*sb;
 		if (!all)
 			break;
 	}
+	/*
+	 * If 'bindir' and 'datadir' are not defined then
+	 * return system configuration value.
+	 */
+	if (!exist) {
+		if (!strcmp(name, "bindir")) {
+			strbuf_puts(sb, BINDIR);
+			exist = 1;
+		} else if (!strcmp(name, "datadir")) {
+			strbuf_puts(sb, DATADIR);
+			exist = 1;
+		}
+	}
 	return exist;
 }
 /*
@@ -421,9 +434,9 @@ STRBUF	*sb;
  */
 int
 getconfb(name)
-const char *name;
+	const char *name;
 {
-	char	buf[MAXPROPLEN+1];
+	char buf[MAXPROPLEN+1];
 
 	if (!opened)
 		openconf();
