@@ -1,8 +1,6 @@
 /*
- * Copyright (c) 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001, 2002
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1998, 1999 Shigio Yamaguchi
+ * Copyright (c) 1999, 2000, 2001, 2002, 2003 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -33,16 +31,19 @@
 #define OLD_GTAGSCONF   "/etc/global.conf"	/* for compatibility */
 #define DEBIANCONF      "/etc/gtags/gtags.conf"
 #define OLD_DEBIANCONF  "/etc/gtags/global.conf"/* for compatibility */
-#define GTAGSRC         ".globalrc"
+#define GTAGSRC 	".globalrc"
+#ifdef __DJGPP__
+#define DOS_GTAGSRC	"_globalrc"
+#endif
 #define DEFAULTLABEL    "default"
-#define DEFAULTSUFFIXES "c,h,y,c++,cc,cpp,cxx,hxx,C,H,s,S,java"
-#define DEFAULTSKIP     "GPATH,GTAGS,GRTAGS,GSYMS,HTML/,tags,TAGS,ID,y.tab.c,y.tab.h,SCCS/,RCS/,CVS/,CVSROOT/,.deps/"
+#define DEFAULTSUFFIXES "c,h,y,c++,cc,cpp,cxx,hxx,hpp,C,H,s,S,java,php,php3,phtml"
+#define DEFAULTSKIP	"GPATH,GTAGS,GRTAGS,GSYMS,HTML/,tags,TAGS,ID,y.tab.c,y.tab.h,.notfunction,cscope.out,.gdbinit,SCCS/,RCS/,CVS/,CVSROOT/,{arch}/,.svn/,.cvsrc,.cvsignore,.cvspass,.cvswrappers,.deps/,autom4te.cache/,.snprj/"
 
-void	openconf(void);
-int	getconfn(const char *, int *);
-int	getconfs(const char *, STRBUF *);
-int	getconfb(const char *);
-char	*getconfline(void);
-void	closeconf(void);
+void openconf(void);
+int getconfn(const char *, int *);
+int getconfs(const char *, STRBUF *);
+int getconfb(const char *);
+char *getconfline(void);
+void closeconf(void);
 
 #endif /* ! _CONF_H_ */
