@@ -19,22 +19,16 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/*
- * If find(1) is available, then use it to traverse directory tree.
- * Otherwise use dirent(3).
- */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 #include <assert.h>
 #include <ctype.h>
-#ifndef HAVE_FIND
+#ifdef HAVE_DIRENT_H
 #include <sys/types.h>
 #include <dirent.h>
-#ifndef HAVE_DP_D_TYPE
 #include <sys/stat.h>
 #endif
-#endif
 #include <stdio.h>
 #ifdef STDC_HEADERS
 #include <stdlib.h>
@@ -321,10 +315,7 @@ skipthisfile(path)
 	}
 	return 0;
 }
-#ifndef HAVE_FIND
-/*----------------------------------------------------------------------*/
-/* dirent version find_xxx()						*/
-/*----------------------------------------------------------------------*/
+
 #define STACKSIZE 50
 static  char dir[MAXPATHLEN+1];			/* directory path */
 static  struct {
@@ -352,36 +343,15 @@ getdirs(dir, sb)
 {
 	DIR *dirp;
 	struct dirent *dp;
-#ifndef HAVE_DP_D_TYPE
 	struct stat st;
-#endif
 
 	if ((dirp = opendir(dir)) == NULL)
 		return -1;
 	while ((dp = readdir(dirp)) != NULL) {
-#ifdef HAVE_DP_D_NAMLEN
-		if (dp->d_namlen == 1 && dp->d_name[0] == '.')
-#else
 		if (!strcmp(dp->d_name, "."))
-#endif
 			continue;
-#ifdef HAVE_DP_D_NAMLEN
-		if (dp->d_namlen == 2 && dp->d_name[0] == '.' && dp->d_name[1] == '.')
-#else
 		if (!strcmp(dp->d_name, ".."))
-#endif
 			continue;
-#ifdef HAVE_DP_D_TYPE
-		if (dp->d_type == DT_DIR)
-			strbuf_putc(sb, 'd');
-		else if (dp->d_type == DT_REG)
-			strbuf_putc(sb, 'f');
-		else if (dp->d_type == DT_LNK)
-			continue;	/* strbuf_putc(sb, 'l'); */
-		else
-			strbuf_putc(sb, ' ');
-		strbuf_puts(sb, dp->d_name);
-#else
 #ifdef HAVE_LSTAT
 		if (lstat(makepath(dir, dp->d_name, NULL), &st) < 0) {
 			fprintf(stderr, "cannot lstat '%s'. (Ignored)\n", dp->d_name);
@@ -399,12 +369,11 @@ getdirs(dir, sb)
 			strbuf_putc(sb, 'f');
 #ifdef S_ISLNK
 		else if (S_ISLNK(st.st_mode))
-			continue;	/* strbuf_putc(sb, 'l'); */
+			strbuf_putc(sb, 'l');
 #endif
 		else
 			strbuf_putc(sb, ' ');
 		strbuf_puts(sb, dp->d_name);
-#endif /* HAVE_DP_D_TYPE */
 		strbuf_putc(sb, '\0');
 	}
 	(void)closedir(dirp);
@@ -461,7 +430,10 @@ find_read(void)
 
 			curp->p += strlen(curp->p) + 1;
 			if (type == 'f' || type == 'l') {
-				char	*path = makepath(dir, unit, NULL);
+				char path[MAXPATHLEN];
+
+				/* makepath() returns unsafe module local area. */
+				strlimcpy(path, makepath(dir, unit, NULL), sizeof(path));
 				if (skipthisfile(path))
 					continue;
 				if (regexec(suff, path, 0, 0, 0) == 0) {
@@ -525,88 +497,3 @@ find_close(void)
 		regfree(skip);
 	opened = 0;
 }
-#else /* !HAVE_FIND */
-/*----------------------------------------------------------------------*/
-/* find command version							*/
-/*----------------------------------------------------------------------*/
-static FILE *ip;
-
-/*
- * find_open: start iterator without GPATH.
- *
- *	i)	start	start directory
- *			If NULL, assumed '.' directory.
- */
-void
-find_open(start)
-	char *start;
-{
-	char findcom[MAXFILLEN+1];
-	assert(opened == 0);
-	opened = 1;
-
-	if (!start)
-		start = ".";
-	if (is_unixy())
-		snprintf(findcom, sizeof(findcom), "find %s \\( -type f -o -type l \\) -print", start);
-	else
-		snprintf(findcom, sizeof(findcom), "find %s -type f -print", start);
-
-#ifdef DEBUG
-	if (debug)
-		fprintf(stderr, "find com: %s\n", findcom);
-#endif
-	/*
-	 * prepare regular expressions.
-	 */
-	prepare_source();
-	prepare_skip();
-	if (!(ip = popen(findcom, "r")))
-		die("cannot execute find.");
-}
-/*
- * find_read: read path without GPATH.
- *
- *	r)		path
- */
-char	*
-find_read(void)
-{
-	static char val[MAXPATHLEN+2];
-	char *path = &val[1];
-	char *p;
-
-	assert(opened == 1);
-	while (fgets(path, MAXPATHLEN, ip)) {
-		/*
-		 * chop(path)
-		 */
-		p = path + strlen(path) - 1;
-		if (*p != '\n')
-			die("output of find(1) is wrong (find_read).");
-		*p = 0;
-		if (skipthisfile(path))
-			continue;
-		if (regexec(suff, path, 0, 0, 0) != 0) {
-			/* other file like 'Makefile' */
-			val[0] = ' ';
-			path = val;
-		}
-		return path;
-	}
-	return NULL;
-}
-/*
- * find_close: close iterator.
- */
-void
-find_close(void)
-{
-	assert(opened == 1);
-	pclose(ip);
-	opened = 0;
-	regfree(suff);
-	if (skip)
-		regfree(skip);
-}
-#endif /* !HAVE_FIND */
