@@ -1,8 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001, 2002
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
+ * Copyright (c) 1999, 2000, 2001, 2002 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -26,14 +24,19 @@
 
 #include "gparam.h"
 #include "abs2rel.h"
+#include "char.h"
 #include "conf.h"
+#include "date.h"
 #include "dbop.h"
 #include "defined.h"
 #include "die.h"
+#include "env.h"
 #include "find.h"
 #include "getdbpath.h"
 #include "gtagsop.h"
 #include "is_unixy.h"
+#include "langmap.h"
+#include "linetable.h"
 #include "locatestring.h"
 #include "makepath.h"
 #include "path.h"
@@ -47,5 +50,7 @@
 #include "token.h"
 #include "usable.h"
 #include "version.h"
+#include "varray.h"
+#include "vfind.h"
 
 #endif /* ! _GLOBAL_H_ */
