@@ -1,8 +1,5 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1999, 2000, 2001 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -25,14 +22,15 @@
 #include <config.h>
 #endif
 #include <stdio.h>
+#ifdef STDC_HEADERS
+#include <stdlib.h>
+#endif
 
 #include "version.h"
 
-extern int vflag;
-
 const char *copy = "\
-Copyright (c) 1996, 1997, 1998, 1999 Shigio Yamaguchi.\n\
-Copyright (c) 1999, 2000, 2001, 2002 Tama Communications Corporation.\n\
+Copyright (c) 1996, 1997, 1998, 1999 Shigio Yamaguchi\n\
+Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004 Tama Communications Corporation\n\
 GNU GLOBAL comes with NO WARRANTY,\n\
 to the extent permitted by law.\n\
 You may redistribute copies of GNU GLOBAL\n\
@@ -40,6 +38,14 @@ under the terms of the GNU General Public License.\n\
 For more information about these matters,\n\
 see the files named COPYING.\n\
 ";
+/*
+ * get_version: get version string.
+ */
+char *
+get_version()
+{
+	return VERSION;
+}
 /*
  * version: print version information.
  */
